/*
---------------------------------------------------------------------
---       EPWIC-1 (Embedded Predictive Wavelet Image Coder)       ---
---    Designed by Robert W. Buccigrossi and Eero P. Simoncelli   ---
---               Developed at the GRASP Laboratory               ---
---          Copyright 1997, University of Pennsylvania           ---
---                      All rights reserved.                     ---
---------------------------------------------------------------------

Permission to use, copy, or modify this software and its documentation
for educational and research purposes only and without fee is hereby
granted, provided that this copyright notice appear on all copies and
supporting documentation.  For any other uses of this software, in
original or modified form, including but not limited to distribution
in whole or in part, specific prior permission must be obtained from
U Penn and the authors.  These programs shall not be used, rewritten,
or adapted as the basis of a commercial software or hardware product
without first obtaining appropriate licenses from U Penn.  U Penn
makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied
warranty.

---------------------------------------------------------------------
*/

#ifndef _ARITH_H
#define _ARITH_H

#include<limits.h>
#include<stdio.h>
#include "epwic.h"  
#include "pyramid_util.h"

/* These routines implement arithmetic coding and were modified from
   those given in "Numerical Recipies in C" to save bits by assuming
   that the receiver knows the length of the string encoded.
   Therefore there is no "terminating symbol" (but one could be
   added) but there is an additional last flag. 
     Also we assume that we are encoding binary information, and
   at every step we pass along the probability of that bit being
   0 (an integer between 0 and 1000 representing a percentage * 10). */


                /* MAXINT is a large positive integer that does not 
                          overflow */
#ifndef MAXINT
 #ifdef ULONG_MAX      /* Maximum value of unsigned long. */ 
  #define MAXINT (ULONG_MAX >> 1) 
 #else 
  #define MAXINT 2147483647 
 #endif 
#endif

#define NWK 2048  /* Number of working longs with which to do the
                   arithmetic coding calculation (the smaller, the more
                   chance of overflow */

/* Structure used to hold important arithmetic coding information */

typedef struct { 
  unsigned long *ilob,   /* Lower bound of probability */
                *iupb,   /* Upper bound of probability */
                jdif,    /* Difference between lower and upper */
                nc,      /* Pointer to beginning of working characters */
                minint, 
                nrad;    /* Radix of encoded stream */ 
  unsigned char *codep;  /* Location for working digits */
  unsigned long lcode;   /* Length of codep */
  unsigned long lcd;     /* Location in codep where we are */
} arithcode;


void nrerror(char error_text[]);
unsigned long *lvector(long nl, long nh);
void free_lvector(unsigned long *v, long nl, long nh);
void arcmak(unsigned long nradd, arithcode *acode);
void arith_encode(unsigned long ich, int prob_zero, int last, 
		  arithcode *acode);
void arith_decode(unsigned long *ich, int prob_zero, arithcode *acode);
int arith_decode_file(FILE *infile, unsigned long *ich, int prob_zero, 
            arithcode *acode);

int arith_encode_byte(unsigned char ich, int last, arithcode *acode);
int arith_encode_block(unsigned char * block, int block_length,
			arithcode *acode);

int arith_decode_file_byte(FILE *infile, unsigned char * ich,
			    arithcode *acode);


void arith_bitplane_init_decode(FILE *infile,Pyramid pyr, 
            int subband, int* read_to_bit, arithcode *acode);
int arith_bitplane_read_bit();
void arith_bitplane_read_byte(Byte *next_symbol);
int arith_bitplane_bytes_read();

#endif /* _ARITH_H */



