/*
---------------------------------------------------------------------
---       EPWIC-1 (Embedded Predictive Wavelet Image Coder)       ---
---    Designed by Robert W. Buccigrossi and Eero P. Simoncelli   ---
---               Developed at the GRASP Laboratory               ---
---          Copyright 1997, University of Pennsylvania           ---
---                      All rights reserved.                     ---
---------------------------------------------------------------------

Permission to use, copy, or modify this software and its documentation
for educational and research purposes only and without fee is hereby
granted, provided that this copyright notice appear on all copies and
supporting documentation.  For any other uses of this software, in
original or modified form, including but not limited to distribution
in whole or in part, specific prior permission must be obtained from
U Penn and the authors.  These programs shall not be used, rewritten,
or adapted as the basis of a commercial software or hardware product
without first obtaining appropriate licenses from U Penn.  U Penn
makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied
warranty.

---------------------------------------------------------------------
*/

#include <stdio.h>
#include <math.h>
#include "matrix.h"
/*
ROUTINES: internal_row_add internal_row_sub internal_row_sum internal_row_swap,
          internal_dot_product, internal_quadratic_form
	  internal_almost_equal, internal_sc_almost_equal, 
	  internal_scalar_multiple,
	  internal_matrix_mul,internal_matrix_mul_transpose,
	  internal_matrix_transpose_mul,internal_matrix_transpose
AUTHOR:	Chichilnisky

PURPOSE:Matrix calculations.
HISTORY:
        butch 9/24/96 made headers and moved defines to matrix.h
*/


/* quadratic form */
float internal_qf_row_arr(vec,arr,rows,cols)
float *arr,*vec;
int rows,cols;
{
  register int row,col;
  register float *p,*q;
  float res = 0.0;

  for (row=rows,p=vec;row;row--,p++) {
    for (col=cols,q=vec;col;col--,q++) {
      res += *arr++ * *p * *q;
    }
  }
  return(res);
}

int internal_qf_rows_arrs(vecs,arrs,res,rows,cols)
float *vecs,*arrs,*res;
int rows,cols;
{
  register int row,cols2=cols*cols;
  register float *p,*q,*r;
  float internal_qf_row_arr();
  
  p = vecs;
  r = res;
  q = arrs;
  for (row=rows;row;row--) {
    *r++ = internal_qf_row_arr(p,q,cols,cols);
    q += cols2;
    p += cols;
  }
  return(0); 
}

int internal_qf_rows_arr(vecs,arr,res,rows,cols)
float *vecs,*arr,*res;
int rows,cols;
{
  register int row;
  register float *p,*r;
  float internal_qf_row_arr();

  p = vecs;
  r = res;
  for (row=rows;row;row--) {
    *r++ = internal_qf_row_arr(p,arr,cols,cols);
    p += cols;
  }
  return(0);
}



double internal_dot_product(arr1,arr2,length)
float *arr1,*arr2;
int length;
{
  register int i;
  double tmp=0.0;

  for (i=length;i;i--) {
    tmp += (*arr1++) * (*arr2++);
  }

  return((double) tmp);
}

int internal_scalar_multiple(float *arr1,float *arr2,float ratio,
			     int length,float tolerance){
  register int i;
  for (i=length;i;i--) {
    if ( fabs((*arr1++) - (ratio * (*arr2++))) > tolerance)
      return(1);
  }
  return(0);
}

int internal_almost_equal(float *arr1,float *arr2,int length,
			  float tolerance){
  register int i;

  for (i=length;i;i--) {
    if ( fabs((*arr1++) - (*arr2++)) > tolerance)
      return(1);
  }
  return(0);
}

int internal_sc_almost_equal(float *arr,float val,int length,
			     float tolerance){
  register int i;
  
  for (i=length;i;i--) {
    if (fabs( (*arr++) - val) > tolerance)
      return(1);
  }
  return(0);
}



int internal_row_swap(array,row1,row2,cols)
float *array;
int row1,row2,cols;
{
  register float *p,*q,tmp;
  register int col;

  p = array + row1*cols;
  q = array + row2*cols;

  for (col=cols;col;col--) {
    tmp = *p;
    *p++ = *q;
    *q++ = tmp;
  }
  return(0);
}


int internal_row_sum_of_square(array,res,rows,cols)
float *array,*res;
int rows,cols;
{
  register float *p=array,*q=res;
  register int row,col;

  for (row=rows;row;row--) {
    *q=0.0;
    for (col=cols;col;col--) {
      *q += *p * *p++;
    }
    q++;
  }
  return(0);
}


/************************************************************************/
/* Arithmetic on rows */

int internal_add_arr_row(arr,vec,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows;row;row--) {
    for (col=cols,p=vec;col;col--) {
      *res++ = *arr++ + *p++;
    }
  }
  return(0);
}

int internal_mul_arr_row(arr,vec,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows;row;row--) {
    for (col=cols,p=vec;col;col--) {
      *res++ = *arr++ * *p++;
    }
  }
  return(0);
}

int internal_sub_arr_row(arr,vec,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows;row;row--) {
    for (col=cols,p=vec;col;col--) {
      *res++ = *arr++ - *p++;
    }
  }
  return(0);
}

int internal_sub_row_arr(vec,arr,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows;row;row--) {
    for (col=cols,p=vec;col;col--) {
      *res++ = *p++ - *arr++;
    }
  }
  return(0);
}

int internal_div_arr_row(arr,vec,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows;row;row--) {
    for (col=cols,p=vec;col;col--) {
      *res++ = *arr++ / *p++;
    }
  }
  return(0);
}

int internal_div_row_arr(vec,arr,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows;row;row--) {
    for (col=cols,p=vec;col;col--) {
      *res++ = *p++ / *arr++;
    }
  }
  return(0);
}



int internal_add_arr_col(arr,vec,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows,p=vec;row;row--) {
    for (col=cols;col;col--) {
      *res++ = *arr++ + *p;
    }
    p++;
  }
  return(0);
}

int internal_mul_arr_col(arr,vec,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows,p=vec;row;row--) {
    for (col=cols;col;col--) {
      *res++ = *arr++ * *p;
    }
    p++;
  }
  return(0);
}

int internal_sub_arr_col(arr,vec,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows,p=vec;row;row--) {
    for (col=cols;col;col--) {
      *res++ = *arr++ - *p;
    }
    p++;
  }
  return(0);
}

int internal_sub_col_arr(vec,arr,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows,p=vec;row;row--) {
    for (col=cols;col;col--) {
      *res++ = *p - *arr++;
    }
    p++;
  }
  return(0);
}


int internal_div_arr_col(arr,vec,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows,p=vec;row;row--) {
    for (col=cols;col;col--) {
      *res++ = *arr++ / *p;
    }
    p++;
  }
  return(0);
}

int internal_div_col_arr(vec,arr,res,rows,cols)
float *arr,*res,*vec;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows,p=vec;row;row--) {
    for (col=cols;col;col--) {
      *res++ = *p / *arr++;
    }
    p++;
  }
  return(0);
}




int internal_row_sum(array,vector,rows,cols)
float *array,*vector;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (col=cols,p=vector;col;col--) {
    *p++ = 0.0;
  }
  for (row=rows;row;row--) {
    for (col=cols,p=vector;col;col--) {
      *p++ += *array++;
    }
  }
  return(0);
}

int internal_col_sum(array,vector,rows,cols)
float *array,*vector;
int rows,cols;
{
  register int row,col;
  register float *p;

  for (row=rows,p=vector;row;row--) {
    *p++ = 0.0;
  }
  for (row=rows,p=vector;row;row--,p++) {
    for (col=cols;col;col--) {
      *p += *array++;
    }
  }
  return(0);
}


int internal_matrix_transpose(m1,r,c,m2)
float *m1,*m2;
int r,c;
{
  register int i,j;
  float tmp;

  if (m1==m2) {
    for(i = 0; i < r; i++) {
      for(j = 0; j < i; j++) {
	tmp = INDEX(m1,j,i,c);
	INDEX(m2,j,i,r) = INDEX(m1,i,j,c);
	INDEX(m2,i,j,r) = tmp;
      }
    }
  }
  else {
    for(i = 0; i < r; i++) {
      for(j = 0; j < c; j++) {
	INDEX(m2,j,i,r) = INDEX(m1,i,j,c);
      }
    }
  }
  return(0);
}



/* Matrix multiplication code speeded up a 11.11.91.
   The old code works fine but does lots of extra multiplication,
   since the new code is uglier, the old is kept below
   in case problems creep in. EJC */

int internal_matrix_mul(m1,r1,c1,m2,r2,c2,m3,r3,c3)
float *m1,*m2,*m3;
int r1,c1,r2,c2,r3,c3;
{
  register int i,j,k;
  register float *p1,*p2,*p3;
  register int off1;

  p3 = m3;
  for (i = r1,off1=0; i ;off1+=c1, i--){
    for (j = 0; j<c2 ; j++){
      p1 = m1 + off1;
      p2 = m2 + j;
      for (k = c1,*p3=0.0; k ; k--) {
	*p3 += *p1++ * *p2;
	p2 += c2;
      }
      p3++;
    }
  }
  return(0);
}



int internal_matrix_mul_transpose(m1,r1,c1,m2,r2,c2,m3,r3,c3)
float *m1,*m2,*m3;
int r1,c1,r2,c2,r3,c3;
{
  register int i,j,k;
  register float *p1,*p2,*p3;
  register int off1,off2;

  p3 = m3;
  for (i = r1,off1=0; i ;off1+=c1, i--){
    for (j = r2,off2=0; j ; off2+=c2, j--){
      p1 = m1 + off1;
      p2 = m2 + off2;
      for (k = c1,*p3=0.0; k ; k--) {
	*p3 += *p1++ * *p2++;
      }
      p3++;
    }
  }
  return(0);
}




int internal_matrix_transpose_mul(m1,r1,c1,m2,r2,c2,m3,r3,c3)
float *m1,*m2,*m3;
int r1,c1,r2,c2,r3,c3;
{
  register int i,j,k;
  register float *p1,*p2,*p3;

  p3 = m3;
  for (i = 0; i<c1 ; i++){
    for (j = 0; j<c2 ; j++){
      p1 = m1 + i;
      p2 = m2 + j;
      for (k = r1,*p3=0.0; k ; k--) {
	*p3 += *p1 * *p2;
	p1 += c1;
	p2 += c2;
      }
      p3++;
    }
  }
  return(0);
}


/************* Old matrix multiplication code *************/
/*
int internal_matrix_mul(m1,r1,c1,m2,r2,c2,m3,r3,c3)
float *m1,*m2,*m3;
int r1,c1,r2,c2,r3,c3;
{
  register int i,j,k;
  float tmp;

  for (i = 0; i < r3; i++){
    for (j = 0; j < c3; j++){
      for (k = 0,tmp=0.0; k < c1; k++) {
	tmp += INDEX(m1,i,k,c1) * INDEX(m2,k,j,c2);
      }
      INDEX(m3,i,j,c3) = tmp;
    }
  }
  return(0);
}


int internal_matrix_transpose_mul(m1,r1,c1,m2,r2,c2,m3,r3,c3)
float *m1,*m2,*m3;
int r1,c1,r2,c2,r3,c3;
{
  register int i,j,k;
  float tmp;

  for (i = 0; i < r3; i++){
    for (j = 0; j < c3; j++){
      for(k = 0,tmp=0.0; k < r1; k++) { 
	tmp += INDEX(m1,k,i,c1) * INDEX(m2,k,j,c2);
      }
      INDEX(m3,i,j,c3) = tmp;
    }
  }
  return(0);
}

int internal_matrix_mul_transpose(m1,r1,c1,m2,r2,c2,m3,r3,c3)
float *m1,*m2,*m3;
int r1,c1,r2,c2,r3,c3;
{
  register int i,j,k;
  float tmp;

  for (i = 0; i < r3; i++){
    for (j = 0; j < c3; j++){
      for(k = 0,tmp=0.0; k < c1; k++) { 
	tmp += INDEX(m1,i,k,c1) * INDEX(m2,j,k,c2);
      }
      INDEX(m3,i,j,c3) = tmp;
    }
  }
  return(0);
}
*/
