/*
---------------------------------------------------------------------
---       EPWIC-1 (Embedded Predictive Wavelet Image Coder)       ---
---    Designed by Robert W. Buccigrossi and Eero P. Simoncelli   ---
---               Developed at the GRASP Laboratory               ---
---          Copyright 1997, University of Pennsylvania           ---
---                      All rights reserved.                     ---
---------------------------------------------------------------------

Permission to use, copy, or modify this software and its documentation
for educational and research purposes only and without fee is hereby
granted, provided that this copyright notice appear on all copies and
supporting documentation.  For any other uses of this software, in
original or modified form, including but not limited to distribution
in whole or in part, specific prior permission must be obtained from
U Penn and the authors.  These programs shall not be used, rewritten,
or adapted as the basis of a commercial software or hardware product
without first obtaining appropriate licenses from U Penn.  U Penn
makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied
warranty.

---------------------------------------------------------------------
*/

/* Matrix routines defined in matrix.c, matrix_inv.c, and svd.c */


/* matrix.c -------------------------------------------------------------- */

/* Make it easy (but slow) to index matrices */
/* If you are feeling gutsy, you can avoid lots of multiplications */

#define INDEX(array,i,j,col) (*(array + ((i)*(col)) + (j)))



/* quadratic form */
float internal_qf_row_arr(float *vec,float *arr,int rows,int cols);

int internal_qf_rows_arrs(float *vecs,float *arrs,float *res,
			  int rows,int cols);

int internal_qf_rows_arr(float *vecs,float *arr,float *res,
			 int rows,int cols);


double internal_dot_product(float *arr1,float *arr2,int length);

int internal_scalar_multiple(float *arr1,float *arr2,float ratio,
			     int length,float tolerance);

int internal_almost_equal(float *arr1,float *arr2,int length,
			  float tolerance);

int internal_sc_almost_equal(float *arr,float val,int length,
			     float tolerance);


int internal_row_swap(float *array,int row1,int row2,int cols);

int internal_row_sum_of_square(float *array,float *res,int rows,int cols);


/************************************************************************/
/* Arithmetic on rows */

int internal_add_arr_row(float *arr,float *vec,float *res,int rows,int cols);

int internal_mul_arr_row(float *arr,float *vec,float *res,int rows,int cols);

int internal_sub_arr_row(float *arr,float *vec,float *res,int rows,int cols);

int internal_sub_row_arr(float *vec,float *arr,float *res,int rows,int cols);

int internal_div_arr_row(float *arr,float *vec,float *res,int rows,int cols);

int internal_div_row_arr(float *vec,float *arr,float *res,int rows,int cols);

int internal_add_arr_col(float *arr,float *vec,float *res,int rows,int cols);

int internal_mul_arr_col(float *arr,float *vec,float *res,int rows,int cols);

int internal_sub_arr_col(float *arr,float *vec,float *res,int rows,int cols);

int internal_sub_col_arr(float *vec,float *arr,float *res,int rows,int cols);

int internal_div_arr_col(float *arr,float *vec,float *res,int rows,int cols);

int internal_div_col_arr(float *vec,float *arr,float *res,int rows,int cols);

int internal_row_sum(float *array,float *vector,int rows,int cols);

int internal_col_sum(float *array,float *vector,int rows,int cols);

int internal_matrix_transpose(float *m1,int r,int c,float *m2);


int internal_matrix_mul(float *m1,int r1,int c1,float *m2,int r2,int c2,
			float *m3,int r3,int c3);

int internal_matrix_mul_transpose(float *m1,int r1,int c1,float *m2,int r2,
				  int c2,float *m3,int r3,int c3);

int internal_matrix_transpose_mul(float *m1,int r1,int c1,float *m2,int r2,
				  int c2,float *m3,int r3,int c3);


/* svd.c------------------------------------------------------------------- */

/* Notes:
   matrix_t is the transposed form of the matrix (and is destroyed), 
   rows, cols are of the original matrix,
   s_1 singular values array with one extra location,
   lsm_t left singular matrix
   rsm_t right singular matrix
   scratchCol size=rows, scratchRow size=cols

  so matrix_t' = lsm_t' * diag(s_1) * rsm_t 
*/

int internal_svd(float *matrix_t,int rows,int cols,float *s_1,float *lsm_t,
		 float *rsm_t,float *scratchCol,float *scratchRow);


/* --- matrix_inv.c ------------------------------------------------------ */
void print_matrix(float*m,int rows, int cols, char *name);

int matrix_inv(float* m, int rows, int cols, float* m_inv);

void least_squares(float *M, int rows, int cols, float*y, float*x);
