/*
---------------------------------------------------------------------
---       EPWIC-1 (Embedded Predictive Wavelet Image Coder)       ---
---    Designed by Robert W. Buccigrossi and Eero P. Simoncelli   ---
---               Developed at the GRASP Laboratory               ---
---          Copyright 1997, University of Pennsylvania           ---
---                      All rights reserved.                     ---
---------------------------------------------------------------------

Permission to use, copy, or modify this software and its documentation
for educational and research purposes only and without fee is hereby
granted, provided that this copyright notice appear on all copies and
supporting documentation.  For any other uses of this software, in
original or modified form, including but not limited to distribution
in whole or in part, specific prior permission must be obtained from
U Penn and the authors.  These programs shall not be used, rewritten,
or adapted as the basis of a commercial software or hardware product
without first obtaining appropriate licenses from U Penn.  U Penn
makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied
warranty.

---------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "matrix.h"

void print_matrix(float*m,int rows, int cols, char *name){
  int r,c;

  printf(" %s = \n",name);
  for(r=0;r<rows;r++){
    for(c=0;c<cols;c++){
      printf("%12.4g ",INDEX(m,r,c,cols));
    }
    printf("\n");
  }
}

/* This calculates the pseudo inverse of the matrix m.  This routine
   is not suitable for LISP since it uses malloc.  It destroys m,
    but you can use the same matrix for m and m_inv */
/* Note: if m=U'*S*V, m'=V'*S*U, and m^-1 = V'*(1/S)*U */

int matrix_inv(float* m, int rows, int cols, float* m_inv){
  int min_row_col,i;
  float *S,*T,*U,*V,*scratchCol,*scratchRow;

  min_row_col = (rows<cols)?rows:cols;

  S = (float *)malloc((min_row_col+1) * sizeof(float));
  T = (float *)malloc(rows*cols*sizeof(float));
  U = (float *)malloc(rows*rows*sizeof(float));
  V = (float *)malloc(cols*cols*sizeof(float));
  scratchCol = (float *)malloc(cols*sizeof(float));
  scratchRow = (float *)malloc(rows*sizeof(float));

  internal_svd(m,cols,rows,S,V,U,scratchCol,scratchRow);

  bzero((void *)m_inv,rows*cols*sizeof(float));

  for(i=0;i<min_row_col;i++){
    if (S[i] != 0.0){
      INDEX(m_inv,i,i,rows) = 1/S[i];  /* Right now, m_inv is full S */
    }
  }
  
  internal_matrix_mul(m_inv,cols,rows,U,rows,rows,T,cols,rows);
  internal_matrix_transpose_mul(V,cols,cols,T,cols,rows,m_inv,cols,rows);

  free(S); free(T); free(U); free(V); free(scratchCol); free(scratchRow);
  return(1);
}

/* Finds min x for || M x - y || ^2 by calculating
    pinv(M' * M) * M' * y */

void least_squares(float *M, int rows, int cols, float*y, float*x){
  float *T,*S;

  T = (float *)malloc(cols*cols*sizeof(float));
  S = (float *)malloc(rows*sizeof(float));

  internal_matrix_transpose_mul(M,rows,cols,y,rows,1,S,cols,1);
  internal_matrix_transpose_mul(M,rows,cols,M,rows,cols,T,cols,cols);
  matrix_inv(T,cols,cols,T);
  internal_matrix_mul(T,cols,cols,S,cols,1,x,cols,1);
}
