/*
---------------------------------------------------------------------
---       EPWIC-1 (Embedded Predictive Wavelet Image Coder)       ---
---    Designed by Robert W. Buccigrossi and Eero P. Simoncelli   ---
---               Developed at the GRASP Laboratory               ---
---          Copyright 1997, University of Pennsylvania           ---
---                      All rights reserved.                     ---
---------------------------------------------------------------------

Permission to use, copy, or modify this software and its documentation
for educational and research purposes only and without fee is hereby
granted, provided that this copyright notice appear on all copies and
supporting documentation.  For any other uses of this software, in
original or modified form, including but not limited to distribution
in whole or in part, specific prior permission must be obtained from
U Penn and the authors.  These programs shall not be used, rewritten,
or adapted as the basis of a commercial software or hardware product
without first obtaining appropriate licenses from U Penn.  U Penn
makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied
warranty.

---------------------------------------------------------------------
*/

/* In pyramid_util.c and pyramid_util.h we define datatypes and routines
   necessary to easily manipulate the quantized wavelet pyramid */

#include <stdlib.h>
#include "pyramid_util.h"

/* pyrNewPyramid takes a list of coefficients, an x_size, a y_size,
   and num_levels and returns a new pyramid.  Note:  do not delete
   the coefficients p after calling this routine */
Pyramid pyrNewPyramid(Coefficient *p, int x_size, int y_size, int num_levels){
  Pyramid pyr;
  int sub;

  pyr = (Pyramid) malloc(sizeof(*pyr));
  pyrNumLevels(pyr) = num_levels;
  pyrNumSubs(pyr) = (num_levels * 3) + 1;
  pyrXSize(pyr) = x_size;
  pyrYSize(pyr) = y_size;
  pyrImSize(pyr) = x_size * y_size;
  pyr->sub_bin_size = (BinValueType*) malloc(pyrNumSubs(pyr)*
					     sizeof(BinValueType));
  pyr->sub_x_size = (int *) malloc(pyrNumSubs(pyr) * sizeof(int));
  pyr->sub_y_size = (int *) malloc(pyrNumSubs(pyr) * sizeof(int));
  pyr->sub_size = (int *) malloc(pyrNumSubs(pyr) * sizeof(int));
  pyr->sub_coefs = (Coefficient **) malloc(pyrNumSubs(pyr) * 
					   sizeof(Coefficient *));

  ForEachSubband(pyr,sub){
    if (sub==0){
      pyrSubXSize(pyr,sub) = x_size >> num_levels;
      pyrSubYSize(pyr,sub) = y_size >> num_levels;
      pyrSubSize(pyr,sub) = pyrSubXSize(pyr,sub) * pyrSubYSize(pyr,sub);
      pyr->sub_coefs[sub] = p;
    } else {
      pyrSubXSize(pyr,sub) = x_size >> (num_levels-((sub-1)/3));
      pyrSubYSize(pyr,sub) = y_size >> (num_levels-((sub-1)/3));
      pyrSubSize(pyr,sub) = pyrSubXSize(pyr,sub) * pyrSubYSize(pyr,sub);
      pyr->sub_coefs[sub] = p + (((sub-1)%3)+1) * pyrSubSize(pyr,sub);
    }
  }

  return(pyr);
}
