/*
---------------------------------------------------------------------
---       EPWIC-1 (Embedded Predictive Wavelet Image Coder)       ---
---    Designed by Robert W. Buccigrossi and Eero P. Simoncelli   ---
---               Developed at the GRASP Laboratory               ---
---          Copyright 1997, University of Pennsylvania           ---
---                      All rights reserved.                     ---
---------------------------------------------------------------------

Permission to use, copy, or modify this software and its documentation
for educational and research purposes only and without fee is hereby
granted, provided that this copyright notice appear on all copies and
supporting documentation.  For any other uses of this software, in
original or modified form, including but not limited to distribution
in whole or in part, specific prior permission must be obtained from
U Penn and the authors.  These programs shall not be used, rewritten,
or adapted as the basis of a commercial software or hardware product
without first obtaining appropriate licenses from U Penn.  U Penn
makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied
warranty.

---------------------------------------------------------------------
*/

/* In pyramid_util.c and pyramid_util.h we define datatypes and routines
   necessary to easily manipulate the quantized wavelet pyramid */

#ifndef _PYRAMID_UTIL_H
#define _PYRAMID_UTIL_H

/* We represent coefficients in sign magnitude form in unsigned shorts
   (which are assumed to be 2 bytes long) */

typedef unsigned short Coefficient;

#define COEF_SIGN_BIT ((Coefficient)(1<<(8*sizeof(Coefficient)-1)))
#define COEF_ABS_MASK (~COEF_SIGN_BIT)

/* Maximum abs value of a coefficient */
#define MAX_ABS_COEFF 1020

/* Number of bits used to represent the abs value of a coefficient */
#define MAX_NUM_BITS  10

/* The type used to represent the binsizes. Should be unsigned. If this is
   changed, be sure to change the places in epwic.c and unepwic.c where 
   binsizes are written or read from files.  */

typedef unsigned short BinValueType;


/* This data structure is used to hold all of the pyramid information
   for convenience */

typedef struct {
  Coefficient   **sub_coefs;
  double        scale_factor;
  int           num_levels;
  int           num_subs;
  int           x_size;
  int           y_size;
  int           im_size;
  BinValueType  *sub_bin_size;
  int           *sub_x_size;
  int           *sub_y_size;
  int           *sub_size;
} *Pyramid;


/* Convenience definitions.  Note that these don't check for
   bounds or NULLS */

/* Gets various elements from the structure */
#define pyrCoefCoord(pyr,sub,x,y)  (pyr->sub_coefs[sub][(y)*(pyr->sub_x_size[sub])+(x)])
#define pyrCoefOffset(pyr,sub,o)   pyr->sub_coefs[sub][o]
#define pyrScaleFactor(pyr)        pyr->scale_factor
#define pyrNumLevels(pyr)          pyr->num_levels
#define pyrNumSubs(pyr)            pyr->num_subs
#define pyrXSize(pyr)              pyr->x_size
#define pyrYSize(pyr)              pyr->y_size
#define pyrImSize(pyr)             pyr->im_size
#define pyrSubBinSize(pyr,sub)     pyr->sub_bin_size[sub]
#define pyrSubXSize(pyr,sub)       pyr->sub_x_size[sub]
#define pyrSubYSize(pyr,sub)       pyr->sub_y_size[sub]
#define pyrSubSize(pyr,sub)        pyr->sub_size[sub]

#define ForEachSubband(pyr,i)      for(i=0;i<pyrNumSubs(pyr);i++)

/* Convenience functions */
/* pyrNewPyramid takes a list of coefficients, an x_size, a y_size,
   and num_levels and returns a new pyramid.  Note:  do not delete
   the coefficients p after calling this routine */
Pyramid pyrNewPyramid(Coefficient *p, int x_size, int y_size, int num_levels);


#endif /* _PYRAMID_UTIL_H */
