/*
---------------------------------------------------------------------
---       EPWIC-1 (Embedded Predictive Wavelet Image Coder)       ---
---    Designed by Robert W. Buccigrossi and Eero P. Simoncelli   ---
---               Developed at the GRASP Laboratory               ---
---          Copyright 1997, University of Pennsylvania           ---
---                      All rights reserved.                     ---
---------------------------------------------------------------------

Permission to use, copy, or modify this software and its documentation
for educational and research purposes only and without fee is hereby
granted, provided that this copyright notice appear on all copies and
supporting documentation.  For any other uses of this software, in
original or modified form, including but not limited to distribution
in whole or in part, specific prior permission must be obtained from
U Penn and the authors.  These programs shall not be used, rewritten,
or adapted as the basis of a commercial software or hardware product
without first obtaining appropriate licenses from U Penn.  U Penn
makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied
warranty.

---------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define DEFAULT_RATE 14400
#define DELAY_POINT 256

void main(int argc, char* argv[]){
  FILE *infile;
  long rate = DEFAULT_RATE;
  int bits_read = 0;
  int c;
  unsigned long delay_time;

  if ((argc < 2) || (strcmp(argv[1],"-h") == 0)){
    printf("usage:\n   %s file_name [bits/sec]\n",argv[0]);
    printf("   The default time is 14400 bits/sec.\n");
    exit(0);
  }

  if (argc >= 3){
    rate = atol(argv[2]);
    if (rate <= 200){
       fprintf(stderr,"%s: illegal rate %s\n",argv[0],argv[2]);
       exit(-1);
    }
  }

  delay_time = DELAY_POINT * 1000000 / rate;

  infile = fopen(argv[1],"r");
  if (infile == NULL){
    fprintf(stderr,"%s: unable to open %s for read\n",argv[0],argv[1]);
    exit(-1);
  }

  while ((c = getc(infile)) != EOF){
    putc(c,stdout);
    fflush(stdout);
    bits_read += 8;
    if (bits_read > DELAY_POINT){
      usleep(delay_time);
      bits_read = 0;
    }
  }
  fclose(infile);
}
