/*
---------------------------------------------------------------------
---       EPWIC-1 (Embedded Predictive Wavelet Image Coder)       ---
---    Designed by Robert W. Buccigrossi and Eero P. Simoncelli   ---
---               Developed at the GRASP Laboratory               ---
---          Copyright 1997, University of Pennsylvania           ---
---                      All rights reserved.                     ---
---------------------------------------------------------------------

Permission to use, copy, or modify this software and its documentation
for educational and research purposes only and without fee is hereby
granted, provided that this copyright notice appear on all copies and
supporting documentation.  For any other uses of this software, in
original or modified form, including but not limited to distribution
in whole or in part, specific prior permission must be obtained from
U Penn and the authors.  These programs shall not be used, rewritten,
or adapted as the basis of a commercial software or hardware product
without first obtaining appropriate licenses from U Penn.  U Penn
makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied
warranty.

---------------------------------------------------------------------
*/

#include <malloc.h>
#include <string.h>
#include "epwic.h"

/* Temporary variables for use in writing/reading files. See epwic.h. */
Byte temp_byte;   
short temp_short;
int temp_int;

char *check_malloc (int size)
  {
  char *the_block;

  the_block = malloc(size);
  if (the_block == NULL)
      {
      printf("\nERROR: unable to allocate sufficient memory!\n");
      exit(-1);
      }
  return(the_block);
  }

void check_free (char *ptr)
  {
  if (ptr != NULL)
    free (ptr);
  }

FILE *check_fopen (char *filename,char * read_write_flag)
  {
  FILE *fp;
  
  fp = fopen(filename, read_write_flag);
  if (fp == NULL) 
      { 
      printf("\nError opening file %s (%s).\n", filename, read_write_flag); 
      exit(-1);
      }
  return(fp);
  }

char *concatenate(char *string1, char *string2)
  {
  char *new_string = check_malloc ( strlen(string1) + strlen(string2) + 1 );

  strcpy (new_string, string1);
  strcat (new_string, string2);
  return (new_string);
  }

/* This will work with bytes, shorts and ints */
void write_array(void *ptr, int ptrsize, int arraysize, FILE* stream){
  int i;
  Byte *p;

  p = (unsigned char *) ptr;
  for (i=0;i<arraysize;i++,p = &(p[ptrsize])){
    if (ptrsize == 1){
      write_byte(*p,stream);
    } else if (ptrsize == 2){
      write_short(*((short*)p),stream);
    } else if (ptrsize == 4){
      write_int(*((int *)p),stream);
    } else {
      fprintf(stderr,"write_array:  Not 1, 2, or 4 bytes\n");
    }
  }
}

/* This will work with bytes, shorts and ints */
void read_array(void *ptr, int ptrsize, int arraysize, FILE* stream){
  int i;
  Byte *p;

  p = (unsigned char *) ptr;
  for (i=0;i<arraysize;i++,p = &(p[ptrsize])){
    if (ptrsize == 1){
      read_byte(*p,stream);
    } else if (ptrsize == 2){
      read_short(*((short *)p),stream);
    } else if (ptrsize == 4){
      read_int(*((int *) p),stream);
    } else {
      fprintf(stderr,"read_array:  Not 1, 2, or 4 bytes\n");
    }
  }
}
